/* Initializer actor initializs the registry of CORBA services which 
 *  will be used by CIPRes software package.
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *                                       PT_COPYRIGHT_VERSION_2
 *                                       COPYRIGHTENDKEY
 */ 

package org.cipres.kepler;

// Ptolemy package
import ptolemy.actor.TypedAtomicActor;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;
import ptolemy.actor.IOPort;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.Token;
import ptolemy.data.StringToken;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;

//////////////////////////////////////////////////////////////////////////
//// Initializer
/**
 * The Initializer actor initializes the Registry of CORBA services
 *  used in CIPRes software package.
 * 
 * @author Zhijie Guan  
 * @version $Id: Initializer.java,v 1.4 2006/05/19 11:51:23 guan Exp $
*/

public class Initializer extends TypedAtomicActor {

	/** Construct an Initializer actor with the given container and name.
	 *  @param container The container.
     *  @param name The name of this actor.
     *  @exception IllegalActionException If the entity cannot be contained
     *   by the proposed container.
     *  @exception NameDuplicationException If the container already has an
     *   actor with this name.
     */
	
    public Initializer(CompositeEntity container, String name)
            throws NameDuplicationException, IllegalActionException  {
        super(container, name);

        // construct the output port outputTrigger
        outputTrigger= new TypedIOPort(this, "outputTrigger", false, true);
        outputTrigger.setDisplayName("Trigger");
        // Set the type constraint.
        outputTrigger.setTypeEquals(BaseType.GENERAL);

		
        _attachText("_iconDescription", "<svg>\n" +
                "<rect x=\"0\" y=\"0\" "
                + "width=\"60\" height=\"20\" "
                + "style=\"fill:white\"/>\n" +
                "</svg>\n");
		
    }

    ///////////////////////////////////////////////////////////////////
    ////                     ports and parameters                  ////

    /** 
     * A trigger signal is sent out through this output port to other 
     *  actors that need to use CIPRes CORBA registry. The signal is sent
     *  out only after the CORBA registry is initialized successfully 
     *  in this actor.
     * This port is an output port of type GENERAL.
     */
    public TypedIOPort outputTrigger = null;

	
	///////////////////////////////////////////////////////////////////
	////                    functional variables                   ////
	
    ///////////////////////////////////////////////////////////////////
    ////                         public methods                    ////

    /** Send out a token to indicate that the intialization is successful.
     *  @exception IllegalActionException If it is thrown by the
     *   send() method sending out the token.
     */
    public void fire() throws IllegalActionException {
        super.fire();
		
		// Here comes the initialization of the facilitator
		// the facilitator should be initialized before any CORBA services is called
		String APPLICATION_NAME = "cipres.application";
		try	{
            org.cipres.communication.Facilitator.initialize(APPLICATION_NAME);
		} catch (Exception e) {
            e.printStackTrace();
		}

		// fill this StringToken with a string message. 
		// Since no actor is going to read this message, (this is only a trigger signal) 
		// it doesn't matter what is put in this message.
		StringToken trigger = new StringToken("Initialization finished!");
		outputTrigger.send(0, trigger);
    }
	
	/** Post fire the actor. Return false to indicated that the
	 *  process has finished. If it returns true, the process will
	 *  continue indefinitely.
	 */
	public boolean postfire() {
		return false;
	}	
}