/**
 *    '$RCSfile: SaveInLibraryDialog.java,v $'
 *
 *     '$Author: bowers $'
 *       '$Date: 2006/05/22 18:37:17 $'
 *   '$Revision: 1.3 $'
 *
 *  For Details: http://kepler-project.org
 *
 * Copyright (c) 2005 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.kepler.ksw.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

import org.kepler.ksw.KSWAlreadyCachedException;
import org.kepler.ksw.KSWBuilder;
import org.kepler.moml.KSWLibraryBuilder;
import org.kepler.objectmanager.ActorMetadata;
import org.kepler.objectmanager.lsid.DuplicateLSIDException;
import org.kepler.objectmanager.lsid.KeplerLSID;
import org.kepler.objectmanager.lsid.LSIDTree;
import org.kepler.objectmanager.lsid.LSIDGenerator;
import org.kepler.objectmanager.cache.*;
import org.kepler.objectmanager.library.LibraryIndex;
import org.kepler.objectmanager.library.LibraryIndexItem;
import org.kepler.objectmanager.library.LibraryIndexComponentItem;
import org.kepler.objectmanager.library.LibraryIndexConceptItem;
import org.kepler.objectmanager.library.LibraryIndexOntologyItem;
import org.kepler.gui.AnnotatedPTree;
import org.kepler.sms.*;
import org.kepler.sms.gui.*;
import org.kepler.moml.NamedObjId;

import ptolemy.actor.TypedCompositeActor;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.ComponentEntity;
import ptolemy.kernel.Entity;
import ptolemy.kernel.util.Attribute;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NamedObj;
import ptolemy.kernel.util.StringAttribute;
import ptolemy.moml.EntityLibrary;

import com.ibm.lsid.LSIDException;




/**
 * @author Chad Berkley
 * @author Shawn Bowers
 *
 * This dialog lets a user store an actor/entity into the actor
 * library. This dialog replaces the add-to-library functionality of
 * the org.kepler.ksw.gui.CreateKSWDialog
 */
public class SaveInLibraryDialog extends JDialog {

    /**
     * Constructor
     * @param owner the Frame that owns this dialog
     * @param entity the entity (actor) being saved
     */
    public SaveInLibraryDialog(Frame owner, Entity entity) {
	super(owner);

	_entity = entity;
	_owner = owner;

	// set title and close behavior
	setTitle("Save in Library");
	setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
	setContentPane(_createPanel());
	setSize(600, 550);
	setResizable(false);

	// check if entity has an lsid and if it exists in the library
	_checkEntity();

	_givenName = _entity.getName();
	_displayNameTxt.setText(_givenName);

	try {
	    _libindex = LibraryIndex.getInstance();
	} catch(Exception e) {
	    e.printStackTrace();
	}

    }


    ////////////////////////////////////////////////////////////////////////////////
    //  PRIVATE METHODS

    /**
     * Create the main panel
     * @returns the created JPanel
     */
    private JPanel _createPanel() {
	// initialize components
	_replaceBox = new JCheckBox("Replace Previous Version in Library");
	_replaceBox.addItemListener(new ReplaceItemListener());

	_disableReplace();

	_classSelector = new OntoClassSelectionJPanel(true, 550, 350);
	_initClassSelector();
	
	_okBtn.setActionCommand("OK");
	_okBtn.setMnemonic(KeyEvent.VK_O);
	_okBtn.addActionListener(buttonListener);
	_cancelBtn.setActionCommand("Cancel");
	_cancelBtn.setMnemonic(KeyEvent.VK_C);
	_cancelBtn.addActionListener(buttonListener);
	
	// overall panel
	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
	panel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	
	// component name label
	JPanel panel1 = new JPanel();
	panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
	panel1.add(_displayNameLbl);
	panel1.add(_displayNameTxt);

	// hide in library check box
	JPanel panel2 = new JPanel();
	panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
	panel2.add(_replaceBox);
	panel2.add(Box.createHorizontalGlue());

	// class selector panel
	JPanel panel3 = new JPanel();
	panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));
	Border eb = BorderFactory.createEtchedBorder();
	Border tb = BorderFactory.createTitledBorder(eb, " Location in Library ");
	((TitledBorder)tb).setTitleColor(Color.gray);
	panel3.setBorder(tb);
	panel3.add(_classSelector);
	panel3.add(Box.createRigidArea(new Dimension(0, 5)));

	// button panel
	JPanel panel4 = new JPanel();
	panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
	panel4.add(Box.createHorizontalGlue());
	panel4.add(_cancelBtn);
	panel4.add(Box.createRigidArea(new Dimension(10, 0)));
	panel4.add(_okBtn);

	// construct overall panel
	panel.add(panel1);
	panel.add(Box.createRigidArea(new Dimension(0, 10)));
	panel.add(panel2);
	panel.add(Box.createRigidArea(new Dimension(0, 15)));
	panel.add(panel3);
	panel.add(Box.createRigidArea(new Dimension(0, 15)));
	panel.add(panel4);

	return panel;
    }

    /**
     * Private method to check if the entity has an lsid, and if it
     * exists in the library.
     */
    private void _checkEntity() {
	// get the entity lsid
	KeplerLSID lsid = _getEntityLSID();
	// if has lsid and it's registered, then okay
	if(lsid != null && LSIDTree.getInstance().isRegistered(lsid))
	    return;

	_replaceBox.setSelected(false);
	_replaceBox.setEnabled(false);
    }

    /**
     * Private method to obtain entity's lsid
     * @returns the KeplerLSID of entity, null otherwise
     */
    private KeplerLSID _getEntityLSID() {
	NamedObjId lsidAtt = (NamedObjId)_entity.getAttribute("entityId");
	if(lsidAtt == null)
	    return null;
	String lsidStr = lsidAtt.getExpression();
	if(lsidStr == null)
	    return null;
	try {
	    return new KeplerLSID(lsidStr);
	} catch(Exception e) {
	    return null;
	}
    }

    /**
     * Private method to assign the given lsid to entity
     * @param lsid the lsid to assign the current entity
     */
    private void _assignEntityLSID(KeplerLSID lsid) {
	try {
	    NamedObjId lsidAtt = (NamedObjId)_entity.getAttribute("entityId");
	    if(lsidAtt == null) 
		lsidAtt = new NamedObjId(_entity, "entityId");
	    lsidAtt.setExpression(lsid.toString());		
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Private method to assign the given name to entity
     * @param name the name to assign the current entity
     */
    private void _assignEntityName(String name) {
	try {
	    _entity.setName(_displayNameTxt.getText());
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Private method to obtain a revision of current lsid
     * @return the revised KeplerLSID, or null if current LSID is null
     */
    private KeplerLSID _createLSIDRevision() {
	if(_getEntityLSID() == null)
	    return null;
	return LSIDTree.getInstance().findNextRevision(_getEntityLSID());
    }

    /**
     * Private method to obtain a brand new lsid
     * @return the newly created, but not registered, KeplerLSID
     */
    private KeplerLSID _createNewLSID() {
	try {
	    KeplerLSID lsid = new KeplerLSID("urn:lsid:localhost:entity:1:1");
	    return LSIDTree.getInstance().findNextObject(lsid);
	} catch(Exception e) {
	    e.printStackTrace();
	}

	return null;
    }

    /**
     * Private method to register a new lsid
     */
    private void _registerLSID(KeplerLSID lsid) {
	try {
	    LSIDTree.getInstance().addLSID(lsid);
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }


    /**
     * Private method to load the class selector with the given
     * ontology classes
     */
    private void _initClassSelector() {
	// get the types
	Vector types = SMSServices.getActorSemanticTypes(_entity);
	// if no types, return
	if(types == null)
	    return;

	Iterator semtypes = types.iterator();
	while(semtypes.hasNext()) {
	    SemanticType st = (SemanticType)semtypes.next();
	    // convert semantic type to NamedOntClass
	    NamedOntClass cls = OntologyCatalog.instance().getNamedOntClass(st);
	    // add to the selector
	    if(cls != null)
		_classSelector.addNamedOntClass(cls);
	}
    }

    /**
     * Private method for performing the save action.
     */
    private void _saveCommand() {
	String msg = "";

	// check if there is a name
	String name = _displayNameTxt.getText();
	if(name == null || name.trim().equals("")) {
	    msg = "Please provide a display name";
	    JOptionPane.showMessageDialog(_owner, msg, "Display Name Missing",
					  JOptionPane.ERROR_MESSAGE);
	    return;
	}

	// check that at least one category is assigned
	Vector types = _classSelector.getNamedOntClasses();
	if(types.size() == 0) {
	    msg = "Please select at least one category";
	    JOptionPane.showMessageDialog(_owner, msg, "Category Assignment Missing",
					  JOptionPane.ERROR_MESSAGE);	    
	    return;
	}

	// check to make sure an actor with the same name doesn't
	if(!_replaceBox.isSelected())
	    if(_libindex.componentNameInUse(name)) {
		msg = "The name '" + name + "' is already in use in the " + 
		    "library. \n Please select another name.";
		JOptionPane.showMessageDialog(_owner, msg, "Duplicate Name",
					      JOptionPane.ERROR_MESSAGE);	    
		return;
	    }
	
	// get the current lsid
	KeplerLSID lsid = _getEntityLSID();


	// create a new or revised lsid
	if(_replaceBox.isSelected()) {
	    // remove the existing item
	    _removeEntity(lsid);
	    // create a revised id
	    lsid = _createLSIDRevision(); 
	}
	else if(lsid == null || !_replaceBox.isSelected()) {
	    lsid = _createNewLSID();
	}

	if(lsid == null) {
	    msg = "Failed to create LSID"; 
	    JOptionPane.showMessageDialog(_owner, msg, "Internal Error",
					  JOptionPane.ERROR_MESSAGE);	    
	    return;	    
	}

	// assign new name to component
	_assignEntityName(_displayNameTxt.getText());
	// assign the new semantic types
	_assignSemTypes();
	// assign new lsid
	_assignEntityLSID(lsid);
	// save the component with the new lsid
	_saveEntity(lsid);
	_close();
    }

    /**
     * Private method for canceling operation
     */
    private void _cancelCommand() {
	_close();
    }

    /** 
     * Private method to close the dialog
     */
    private void _close() {
	setVisible(false);	
	dispose();
    }

    /**
     * Private method for assigning the selected semantic types
     */
    private void _assignSemTypes() {
	    // remove the existing types
	    Vector semtypes = new Vector();
	    Iterator iter = _entity.attributeList(SemanticType.class).iterator();
	    while(iter.hasNext())
		semtypes.add((SemanticType)iter.next());
	    iter = semtypes.iterator();
	    while(iter.hasNext()) {
		SemanticType st = (SemanticType)iter.next();
	    try {
		st.setContainer(null);
	    } catch(Exception e) {
		e.printStackTrace();
	    }
	}
	
	int cnt = 0;
	Vector types = _classSelector.getNamedOntClasses();
	for(int i = 0; i < types.size(); i++) {
	    NamedOntClass cls = (NamedOntClass)types.elementAt(i);
	    try { 
		SemanticType st = new SemanticType(_entity, "semanticType" + i);
		st.setConceptId(cls.getConceptId());
	    } catch(Exception e) {
		e.printStackTrace();
	    }
	}
    }

    /**
     * Private method to remove the entity from from the library index
     * (model) and the actorlibrary component (view)
     */
    private void _removeEntity(KeplerLSID lsid) {
	if(lsid == null)
	    return;
	try {
	    _libindex.removeAll(lsid);
	    // _libindex.rebuild();
	    // _libindex.refresh();
	    _removeFromActorLibrary(lsid);
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Private method for removing each component with the given lsid
     * from the actorlibrary (view) 
     */
    private void _removeFromActorLibrary(KeplerLSID lsid) {
	if(lsid == null)
	    return;

	try {
	    // get the current actor library
	    CompositeEntity actorLibrary = _libindex.getActorLibrary();
	    // get the items in library of the entity class
	    Iterator iter = actorLibrary.entityList().iterator();
	    while(iter.hasNext()) {
		NamedObj obj = (NamedObj)iter.next();
		// get the entityid attribute
		NamedObjId lsidAtt = (NamedObjId)obj.getAttribute("entityId");	    
		if(lsidAtt != null && lsid.getLsid().equals(lsidAtt.getExpression())) {
		    if(obj instanceof Attribute)
			((Attribute)obj).setContainer(null);
		    else if(obj instanceof ComponentEntity)
			((ComponentEntity)obj).setContainer(null);

		    // issue a change reqest
		    util.EmptyChangeRequest request = 
			new util.EmptyChangeRequest(obj, "request change");
		    obj.requestChange(request);
		}
	    }
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }


    /**
     * Private method to save the entity to the cache, and also update
     * the actor library 
     */
    private void _saveEntity(KeplerLSID lsid) {
	try {
	    // create a new actor metadata entry
	    ActorMetadata am = new ActorMetadata(_entity);
	    am.setName(_entity.getName());
	    am.setId(lsid.toString());

	    // add actor metadata to the cache
	    InputStream instream = new ByteArrayInputStream(am.toString().getBytes());
	    ActorCacheObject aco = new ActorCacheObject(instream);
	    CacheManager cache = CacheManager.getInstance();
	    cache.insertObject(aco);

	    // check if we have a handle to the actor library
	    CompositeEntity actorLibrary = _libindex.getActorLibrary();
	    if(actorLibrary == null)
		return;

	    NamedObj actorEntity = am.getActorAsNamedObj(actorLibrary);

	    // add to each tree location
	    Iterator types = _classSelector.getNamedOntClasses().iterator();
	    while(types.hasNext()) {
		NamedOntClass cls = (NamedOntClass)types.next();
		LibraryIndexItem subtree = _getClass(cls.getConceptId());
		if(subtree != null) {
		    LibraryIndexComponentItem item = 
			new LibraryIndexComponentItem(am.getName(), am.getLSID());
		    _libindex.add(item, subtree);
		}
	    }

	    // rebuild and refresh the library
	    _libindex.rebuild();
	    _libindex.refresh();
	    // change request on the new entity in the library 
	    util.EmptyChangeRequest request = 
		new util.EmptyChangeRequest(actorEntity, "request change");
	    actorEntity.requestChange(request);
	    // change request on the old entity on the canvas
	    request = new util.EmptyChangeRequest(_entity, "request change");
	    _entity.requestChange(request);

	} catch(Exception e) {
	    e.printStackTrace();
	}

    }


    /**
     * Private method to obtain the item with the given conceptId
     * @returns the item or null if no such item exists
     */
    private LibraryIndexItem _getClass(String conceptId) {
	if(conceptId == null)
	    return null;

	Iterator iter = null;
	try {
	    iter = _libindex.concepts(null);
	} catch(Exception e) {
	    e.printStackTrace();
	    return null;
	}

	while(iter.hasNext()) {
	    LibraryIndexConceptItem item = (LibraryIndexConceptItem)iter.next();
	    if(conceptId.equals(item.getConceptId()))
	       return item;
	}
	return null;
    }

    /**
     * Private method for disabling the replace check box
     */
    private void _disableReplace() {
	_displayNameTxt.setText(_givenName);
	_displayNameTxt.setEnabled(true);
	_displayNameLbl.setEnabled(true);	
    }

    /**
     * Private method for enabling the replace check box
     */
    private void _enableReplace() {
	_displayNameTxt.setText(_entity.getName());
	_displayNameTxt.setEnabled(false);
	_displayNameLbl.setEnabled(false);
    }


    ////////////////////////////////////////////////////////////////////////////////
    // PRIVATE INNER CLASSES


    /**
     * Anonymous class to handle button events
     */
    private ActionListener buttonListener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("OK"))
		    _saveCommand();
		else if(e.getActionCommand().equals("Cancel")) 
		    _cancelCommand();
	}

    }; 


    /**
     * Private class for reacting to changes in the replace check box
     */
    private class ReplaceItemListener implements ItemListener {
	/** 
	 * When replace box is checked, deactivates display name
	 */
	public void itemStateChanged(ItemEvent e) {
	    Object source = e.getItemSelectable();
	    if(source == _replaceBox) {
		if(e.getStateChange() == ItemEvent.DESELECTED)
		    _disableReplace();
		else if(e.getStateChange() == ItemEvent.SELECTED) {
		    // remember current name and enable
		    _givenName = _displayNameTxt.getText(); 
		    _enableReplace();
		}
	    }
	}
    };


    ////////////////////////////////////////////////////////////////////////////////
    // PRIVATE MEMBERS

    private JButton _okBtn = new JButton("OK");
    private JButton _cancelBtn = new JButton("Cancel");
    private JCheckBox _replaceBox;
    private JLabel _displayNameLbl = new JLabel("Display Name: ");
    private JTextField _displayNameTxt = new JTextField(12);
    private OntoClassSelectionJPanel _classSelector;
    private Entity _entity;
    private Frame _owner;
    private String _givenName = "";
    private LibraryIndex _libindex;



    ////////////////////////////////////////////////////////////////////////////////
    // TESTING


    /**
     * Main method for testing the dialog.
     *
     *@param  args  the arguments to the program
     */
    public static void main(String[] args)
    {
	try {
	    // a composite "wrapper"
	    TypedCompositeActor swf = new TypedCompositeActor();
	    swf.setName("Fancy Output Handler");
	    String conceptId = "";
	    SemanticType st1 = new SemanticType(swf, "_semanticType");
	    st1.setConceptId("urn:lsid:localhost:onto:1:1#DataOutput");
		
	    // windows look and feel
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		
	    SaveInLibraryDialog dialog = new SaveInLibraryDialog(null, swf);
		
	    dialog.setVisible(true);
	}
	catch(Exception e) {
	    e.printStackTrace();
	}
    }

}    

